
drop table equipment cascade constraints;
drop table customer cascade constraints;
drop table armory cascade constraints;
drop table rental cascade constraints;
drop table located cascade constraints;
drop sequence confirmations; 

/* Our equipment table - the primary key is 'id'  */

create table equipment (
	id		varchar2(256) not null,
	title		varchar2(40) not null,
	imageURL 	varchar2(512) not null,
	thumbnailURL	varchar2(512) not null,
	shortSummary	varchar2(80) not null,
	summary		varchar2(4000),
	perWeek		number(10,2) not null,
	perDay		number(10,2) not null,
	type		varchar2(4) not null,
	primary key (id),
	check(perWeek>=0),
	check(perDay>=0)
);

/* Our customer table - the primary key is 'email' */

create table customer (
	email		varchar2(256) not null,
	fname		varchar2(15) not null,
	lname		varchar2(40) not null,
	telephone	varchar2(50) not null,
	primary key (email)
);

/* Our armory - our locations table with the primary key being the 'name' */

create table armory (
	name		varchar2(100) not null,
	country		varchar2(50) not null,
	city		varchar2(100) not null,
	telephone	varchar2(50) not null,
	address	 	varchar2(200) not null,
	primary key (name)
);

/* Our rental table -  which contains a lots of foreign keys */

create table rental (
	confirmationid	number(14,0) not null,
	price		number(14,2) not null,
	pick_up_date	date not null,
	pick_up_hour	varchar2(4), 
	pick_up_armory	varchar2(100) not null,
	drop_off_date	date not null,
	drop_off_hour	varchar2(4),
	drop_off_armory	varchar2(200) not null,
	customer_email	varchar2(256) not null,
	equipment_id	varchar2(256) not null,
	primary key (confirmationid),
        foreign key (pick_up_armory) references armory(name), 
        foreign key (drop_off_armory) references armory(name),
        foreign key (customer_email) references customer(email),
        foreign key (equipment_id) references equipment(id) 
);

/* Location and armory during the drop-off and pick-up */
/* Tied in with the confirmation ID */

create table located (
	confirmationid	number(14,0) not null,
	equipment_id	varchar2(256) not null,
	armory_name	varchar2(100) not null,
	foreign key	(confirmationid) references rental(confirmationid), 
	foreign key	(equipment_id) references equipment(id),
	foreign key	(armory_name) references armory(name)
);

/* Our sequence table used to generate confirmation id */

create sequence confirmations start with 10000;

exit; 
