package militaryrental;

import java.sql.Date;
import java.util.StringTokenizer;

/**
 * This simple Bean contains information about the customer
 * and his choices.
 */

public class Customer {

  private String[] months = {"","Jan","Feb","Mar","Apr","May","June","July","Aug","Sept","Oct","Nov","Dec"};
  private int[] days_in_mnth = {0,31,28,31,30,31,30,31,31,30,31,30,31};

  /**
   * Initialize the Customer with some predefined values.
   */
  public Customer(long id, double price, 
    		       Date p_date , String p_hour, String p_armory,
		       Date d_date , String d_hour, String d_armory,
		       String email, String fname, String lname,
		  String telephone, String equipment_id) {

      this.id = equipment_id;
      this.fname = fname;
      this.lname = lname;
      this.email = email;
      this.telephone = telephone;
      this.price = price;
      this.confirmation_id = id;
      this.country_from = p_armory;
      this.country_to = d_armory;
      this.p_date = p_date;
      this.d_date = d_date;
      this.hour_to = d_hour;
      this.hour_from = p_hour;      
      
      // extract from java.sql.Date the dates

      StringTokenizer st = new StringTokenizer(p_date.toString(),"-");
      this.year_from = st.nextToken();
      this.month_from = st.nextToken();
      this.day_from = st.nextToken();

      st = new StringTokenizer(d_date.toString(), "-");
      this.year_to = st.nextToken();
      this.month_to = st.nextToken();
      this.day_to = st.nextToken();

      //      System.out.println("From: D(" + day_from + "), M("+month_from+"), Y("+year_from+")");
      //System.out.println("To: D(" + day_to + "), N("+month_to +"), Y("+year_to+")");

  }

    /**
     * This is used when we initialize when the customer firsts visits the 
     * the webpage.
     */
    public Customer() { }


    /*******************************************************
     *    These methods and queries conform to the set/get *
     *    conformity used in JavaBeans                     *
     */


  /**
   * The equipment we want to rent.
   */
  public void setEquipmentId(String newValue) {

    if (newValue!=null)
      id = newValue;
  }

  public String getEquipmentId() {

    return id;
  }

  /**
   * Personal info
   */
  public void setFirstName(String newValue) {

    if (newValue!=null)
      fname = newValue;

  }
  public String getFirstName() {
    return fname;
  }

  public void setLastName(String newValue) {

    if (newValue!=null)
      lname = newValue;
  }

  public String getLastName() {

    return lname;
  }

  public void setTelephone(String newValue) {

    if (newValue!=null)
      telephone = newValue;

  }

  public String getTelephone() {

    return telephone;
  }

  public void setEmail(String newValue) {

    if (newValue!=null)
      email = newValue;

  }

  public String getEmail() {

    return email;
  }

    /**
     * The country (armory) where the customer drops off 
     * the equipment.
     */
  public void setCountryTo(String newValue) {

    if (newValue!=null)
      country_to = newValue;
  }

  public String getCountryTo() {

    return country_to;

  }

    /**
     * The country (armory) where hte customer picks up
     * the equipment. 
     *
     */
  public void setCountryFrom(String newValue) {

    if (newValue!=null)
	country_from = newValue;

   }

  public String getCountryFrom() {

    return country_from;
  }


    /**
     * Month in which the customer will drop off the 
     * equipment.
     * 
     * require:
     *   newValue must be "Jan", "Feb" ... "Dec" type string
     */
  public void setMonthTo(String newValue) {

    if (newValue!=null) {
        month_to = newValue;
        this.setShortMonthTo(figureOutMonth(newValue));
    }
  }

  public String getMonthTo() {
    return month_to;
  }

  public void setMonthFrom(String newValue) {

      if (newValue!=null) {
          month_from = newValue;
          this.setShortMonthFrom(figureOutMonth(newValue));
      }
  }

    /**
     * Sets the year in which the customer will drop off
     * the equipment.
     *
     * require:
     *   The newValue must be of the form: "2002","2003", ... 
     */
  public void setYearFrom(String newValue) {

      if (newValue != null) {
          year_from = newValue;
      }
  }

  public String getYearFrom() {
    return year_from;
  }

  public void setYearTo(String newValue) {

      if (newValue != null) {
          year_to = newValue;
      }
  }

  public String getYearTo() {
    return year_to;
  }

  public String getMonthFrom() {
    return month_from;
  }


    /**
     * Set the month in which the customer will drop-off
     * the equipment.
     *
     * require:
     *  the newValue must be: 1..12
     */
  public void setShortMonthTo(int newValue) {
    s_month_to = newValue;
    month_to = months[newValue];
  }

  public int getShortMonthTo() {
    return s_month_to;
  }

    /**
     * Set the month in which the customer will pick-up
     * the equipment.
     *
     * require:
     *  the newValue must be 1..12
     */
  public void setShortMonthFrom(int newValue) {
    s_month_from = newValue;
    month_from = months[newValue];
  }

  public int getShortMonthFrom() {
    return s_month_from;
  }

    /**
     * Set the day in which the customer will drop off tthe
     * the equipment.
     *
     * require:
     *  the newValue must be 1..31
     */
  public void setDayTo(String newValue) {
    if (newValue!=null)
      day_to = newValue;
  }

  public String getDayTo() {
    return day_to;
  }

  public void setDayFrom(String newValue) {
    if (newValue!=null)
      day_from = newValue;
  }

  public String getDayFrom() {
    return day_from;
  }


    /**
     * Retrives the java.sql.Date object with the 
     * pick-up date (year-month-day)
     */
    public Date getDateFrom() {
        p_date = Date.valueOf(year_from + "-" + figureOutMonth(month_from) + "-"+day_from);
	//	System.out.println(p_date);
	return p_date;
    }
 
    /**
     * Retrives the java.sql.Date object with the
     * drop-off date (year-month-day)
     */
    public Date getDateTo() {
	d_date = Date.valueOf(year_to + "-" + figureOutMonth(month_to) + "-" + day_to);
	//	System.out.println(d_date);
	return d_date;
    }

    /**
     * Sets the hour in which the customer wishes to 
     * pick-up the equipment
     */
  public void setHourTo(String newValue) {
    if (newValue!=null)
      hour_to = newValue;
  }

  public String getHourTo() {
    return hour_to;
  }

  public void setHourFrom(String newValue) {
    if (newValue!=null)
      hour_from = newValue;
  }

  public String getHourFrom() {
    return hour_from;
  }


  /**
   * Calculate how many days the equipment is going to be used for.
   */
  public int getTotalDays() {
      
    int s_day_from = -1;
    int s_day_to = -1;
    int years  = 0;
    days = -3;
    try {
      years = Integer.parseInt(year_to) - Integer.parseInt(year_from);
      s_day_from = Integer.parseInt(day_from);
      s_day_to = Integer.parseInt(day_to);    
      s_month_to = figureOutMonth(month_to);
      s_month_from = figureOutMonth(month_from);
      //      System.out.println("[ days:"+day_from+" -> "+s_day_from +"; "+day_to+" -> "+s_day_to+"]");
      //System.out.println("{ mmonths:"+s_month_from+" -> "+s_month_to+"}");
      //System.out.println("[ yrs::"+years+"]");
    } catch (Exception e) { 
        s_day_from = s_day_to = -1; 
	s_month_to  = s_month_from = -1;	
    }

    
    days = 365 * years;    

    if ((s_month_to <= 0) || (s_month_from <= 0) || (s_day_from <= 0) || (s_day_to <= 0))
      return days;


    if (s_month_to == s_month_from) {
      days = days + (s_day_to - s_day_from);
    }
    else
    {
      // In the from month, we calculate how many days until the end of the month.
      days = days + days_in_mnth[s_month_from]-s_day_from;
      // This is the end date. Just add it.
      days = days + s_day_to;

      // And the month between these two dates, excluding the to/from months
      for (int ii=s_month_from+1; ii<s_month_to; ii++)
        days += days_in_mnth[ii];
    }
    return days;
  }

    /**
     * Find out how many weeks the equipment is being used
     * for.
     *
     * return:
     *  0 - 52 weeks
     */
  public int getWeeks() {

    return (int)(getTotalDays() / 7);

  }

    /**
     * Find out how many days not included in the weeks
     * are being used for the equipment.
     * 
     * return:
     *  0 - 6
     */
  public int getDays() {

    return (getTotalDays() % 7);

  }


  /**
   * Figure the month in a decimal format (1-12) from a string format (Jan-Dec)
   */
  private int figureOutMonth(String month) {

   
    int result = 0;

    // Loop and quit when the match is found. If match is not found, the last item is picked - not good.
    for ( result=0; (result<months.length) && (month.toLowerCase().compareTo(months[result].toLowerCase()) != 0); result++);

    return result;
  }

    private int days = -1;
    private String country_to = "";
    private String country_from = "";
    private String month_to = "Jan";
    private String month_from = "Jan";
    private int s_month_to = 0;
    private int s_month_from = 0;
    private String day_to = "1";
    private String year_from = "2001";
    private String year_to = "2001";
    private String day_from = "1";
    private String hour_to = "1200";
    private String hour_from = "1200";
    private String fname = "";
    private String lname = "";
    private String telephone = "";
    private String email = "";
    private String id = "";
    private Date p_date;
    private Date d_date;
    private double price;
    private long confirmation_id;
}
