package militaryrental;

import java.util.*;
import oracle.jdbc.driver.OracleDriver;
import java.sql.*;


public class Fleet {

    private Connection con = null;
    /* Our DB connection */
    
    //private String dbName    = "jdbc:oracle:thin:konrad/A@10.0.0.4:1521:darnok";
    private String dbName    = "jdbc:oracle:thin:krzeszut/konrad15@cook.cs.uno.edu:1521:cs4125";
    /* Our SQL statements */

    /* Select all the attributes from the EQUIPMENT table */
    private String selectStr = "select * from equipment";

    /* Select only the records which match the type */
    private String selectByTypeStr = "select * from equipment where type = ?";

    /* Select only the record which id matches */
    private String selectByIdStr = "select * from equipment where id = ?";
    
    PreparedStatement selectStmt = null;
    PreparedStatement selectByTypeStmt = null;
    PreparedStatement selectByIdStmt = null;


    /**
     * Our constructor.
     */
    public Fleet() {
	
	this.initConnection();
	
    }
    
    
    /**
     * Receive the count of total equipment.
     *
     * return:
     * >=0 
     */
    public int getNumberOfEquipment() {
	
	if (equipments == null) 
	    return getEquipments().size();
	
	return equipments.size();
	
    }
    
    /**
     * Receives a Collection object consiting of EquipmentDetail objects.
     * Each EquipmentDetail object has information pertraining to a specific
     * equipment.
     *
     * return:
     *  != null
     */
    public Collection getEquipments() {
	
	if (equipments != null)
	    return equipments;
	
	equipments = new ArrayList();
	
	if (con == null)
	    initConnection();
	try {
	    ResultSet rs = selectStmt.executeQuery();	  
	  while (rs.next()) {
	      System.out.println("["+rs.getString(2)+"]");	 
	      EquipmentDetail bd = new EquipmentDetail(rs.getString(1), rs.getString(2), rs.getString(3), rs.getString(4),
						       rs.getString(5), rs.getString(6), rs.getDouble(7), rs.getDouble(8));
	      equipments.add(bd);
	  }
	  
	} catch (SQLException ex) {
	    ex.printStackTrace();      
	}
	
	Collections.sort(equipments);
	return equipments;
      
    }
    
    /**
     * Retrieve a list of equipments based on a particular type
     *
     * return:
     *  != null
     */
    public Collection getEquipmentsByType(String type) {
	
	if (con == null)
	    initConnection();
	
	ArrayList equipments_by_type = new ArrayList();
	
        try {
	    
	    selectByTypeStmt.setString(1,type);
	    ResultSet rs = selectByTypeStmt.executeQuery();		     
	    while (rs.next()) {
		System.out.println("Type:"+type+", ["+rs.getString(2)+"]");
		EquipmentDetail bd = new EquipmentDetail(rs.getString(1), rs.getString(2), rs.getString(3), rs.getString(4),
							 rs.getString(5), rs.getString(6), rs.getDouble(7), rs.getDouble(8));
		equipments_by_type.add(bd);
	    }
	    
	} catch (SQLException ex) {
	    ex.printStackTrace();
	}
	Collections.sort(equipments_by_type);
	return equipments_by_type;
	
    }

    /**
     * Retrive specific information about a particular equipment.
     *
     * return
     * != null
     */
    public  EquipmentDetail getEquipmentDetail(String equipment_id) {
	
	if (con == null)
	    initConnection();
        try {
	    selectByIdStmt.setString(1,equipment_id);
	    ResultSet rs = selectByIdStmt.executeQuery();
	    
	    while (rs.next()) {
		EquipmentDetail bd = new EquipmentDetail(rs.getString(1), rs.getString(2), rs.getString(3), rs.getString(4),
							 rs.getString(5), rs.getString(6), rs.getDouble(7), rs.getDouble(8));
		
		return bd;
	    }
	} catch (SQLException ex) {
	    ex.printStackTrace();
	}
	return new EquipmentDetail();
    }
    
    /**
     * Make the DB connection and init the PrepareStatement
     */
    private void initConnection() {
	try {
	    Class.forName("oracle.jdbc.driver.OracleDriver");
	} catch (Exception e) { e.printStackTrace(); }
	
	try {
	    con = DriverManager.getConnection (dbName);
	    selectStmt = con.prepareStatement(selectStr);
	    selectByTypeStmt = con.prepareStatement(selectByTypeStr);
	    selectByIdStmt = con.prepareStatement(selectByIdStr);
	} catch (SQLException e) {
	    e.printStackTrace();
	}
    }    
    private ArrayList equipments;
    
}
