package militaryrental;


import oracle.jdbc.driver.OracleDriver;
import java.sql.*;
import java.util.*;

public class Reservations {

    Connection con = null;
    /* Our DB connection - change it accordingly */
    
    //private String dbName  = "jdbc:oracle:thin:konrad/A@10.0.0.4:1521:darnok";
    private String dbName  = "jdbc:oracle:thin:krzeszut/konrad15@cook.cs.uno.edu:1521:cs4125";

    /* Our SQL statements */

    /* Insert all the values into the CUSTOMER table */
    private String insertCustomer = "insert into  customer values (?,?,?,?)";
    /* And here we are using our SEQUENCE table */
    private String insertRental = "insert into rental values (confirmations.nextval,?,?,?,?,?,?,?,?,?)";

    /* Update the CUSTOMER table with the customer e-mail */
    private String updateCustomer = "update customer set fname = ? , lname = ?, telephone = ? where email = ?";

    /* Find the custmer with a specific e-mail */
    private String selectByEmail = "select * from customer where email = ?";

    /* Find a specific Reservation with a specific confirmationid */
    private String selectByID = "select * from rental where confirmationid = ?";

    /* Find the confirmation id depedeing on the customer's e-mail, equipment_id and drop/pick up dates */
    private String selectByRest = "select confirmationid from rental where customer_email = ? and equipment_id = ? and drop_off_date = ? and pick_up_date = ?";

    /* Delete a Reservation based on the confirmaiton id */
    private String deleteByID = "delete from rental where confirmationid = ?";

    /* Insert a location for the equipment and the armory */
    private String insertLocation = "insert into located values (?,?,?)";

    /* Remove a the euqipment from the armory */
    private String deleteLocation = "delete from located where confirmationid = ?";

    /* Find customer information where the e-mail address is used as a search argument */ 
    private String findAll = "select confirmationid, equipment_id,fname, lname, email, pick_up_armory, pick_up_date, drop_off_armory, drop_off_date, price "+
	"from rental, customer"+
	" where rental.customer_email = customer.email";


    private String sumString = "select sum(price) from rental";
    private String averageString = "select avg(price) from rental";
    private String maxString = "select confirmationid, equipment_id, fname, lname, email, telephone from rental, customer where rental.customer_email = customer.email group by price";

    private PreparedStatement sumStmt = null;
    private PreparedStatement averageStmt = null;
    private PreparedStatement maxStmt = null;
    private PreparedStatement insertCustomerStmt = null;
    private PreparedStatement updateCustomerStmt = null;
    private PreparedStatement findCustomerByEmailStmt = null;
    private PreparedStatement findRentalByID = null;
    private PreparedStatement deleteRentalByID = null;
    private PreparedStatement insertRentalStmt = null;
    private PreparedStatement findConfirmationIDByRest = null;
    private PreparedStatement insertLocationStmt = null;
    private PreparedStatement deleteLocationStmt = null;
    private PreparedStatement findAllStmt = null;

    /**
     * Inits our object 
     */
    public Reservations()  throws SQLException{

    	initConnection();
    }

    /**
     * Makes a reservation using the Customer bean to pull 
     * the customer's info and the Fleet to compute the 
     * rental price.
     */
    public ReservationStatus makeReservation(Customer customer, Fleet fleet) throws SQLException 
    {
	
	ReservationStatus status = new ReservationStatus(false,0,"Unknown");
	long confirm_id;

	if (customerExist(customer.getEmail())) {
	    customerUpdate(customer);
	} else
	    customerInsert(customer);
	

	// Check to see if the reservation is already in the DB.
	confirm_id = reservationExist(customer);
	if (confirm_id > 0) {
	    status = new ReservationStatus(false,confirm_id,
					   "You already have a reservation for this particular equipment ("+
					   customer.getEquipmentId() + ") on "+
					   "<b>" + customer.getDateFrom() +"</b> (from "+ customer.getCountryFrom() +")"+
					   " till "+
					   "<b>" + customer.getDateTo() + "</b> (to "+customer.getCountryTo() + ") !<br><br>" +
					   "The existing reservation id is <b>"+confirm_id+"</b>.");
	}
        else 
	    status = insertReservation(customer, fleet);
	
	/* And lastly we connect LOCATED table */
	
	insertLocationStmt.setLong(1, status.getConfirmationId());
	insertLocationStmt.setString(2, customer.getEquipmentId());
	insertLocationStmt.setString(3, customer.getCountryFrom());
	
	insertLocationStmt.executeUpdate();
	
	insertLocationStmt.setString(3, customer.getCountryTo());
	insertLocationStmt.setString(2, customer.getEquipmentId());
	insertLocationStmt.setLong(1, status.getConfirmationId());
	
	insertLocationStmt.executeUpdate();

	return status;
    }
    
    /**
     * Retrieves a Customer object based on the confirmation ID.
     *
     * return 
     * !null
     */
    
    public Customer getCustomerReservationById(long id) throws SQLException {
	
	Customer c = null;
	findRentalByID.setLong(1, id);
	ResultSet rs = findRentalByID.executeQuery();
	if (rs.next()) {
	    c = new Customer(id, rs.getDouble(2), 
			     rs.getDate(3), rs.getString(4),rs.getString(5),
			     rs.getDate(6), rs.getString(7),rs.getString(8),
			     rs.getString(9), "", "",
			     "",rs.getString(10));
	    
	    findCustomerByEmailStmt.setString(1, rs.getString(9));
	    ResultSet rs2 = findCustomerByEmailStmt.executeQuery();
	    if (rs2.next()) {
		c.setFirstName(rs2.getString(2));
		c.setLastName(rs2.getString(3));
		c.setTelephone(rs2.getString(4));
	    }
	    rs2.close();
	    rs.close();
	}
	return c;
    }
   
    /**
     * Deletes a reservation from the DB rental system based on
     * the given confirmation id.
     *
     * requires:
     *  id > 0
     * return:
     *  != null
     */
    public ReservationStatus deleteReservation(long id) throws SQLException {
	
	ReservationStatus status = null;
	int result = 0;
	
	// Just to make sure that all the updates have been commited.
	con.commit();
	
	deleteLocationStmt.setLong(1, id);
	deleteLocationStmt.executeUpdate();
	
	deleteRentalByID.setLong(1, id);	
	result = deleteRentalByID.executeUpdate();
	
	// BOOM and our record is deleted. The 'result' will confirm this.
	if (result > 0) 
	    status = new ReservationStatus(true,id,"Deleted the reservation <b>"+id+"</b> successfully.");
	else
	    status = new ReservationStatus(false,id,"Reservation <b>"+id+"</b> doesn't exist!");
	return status;
    }
    
    
    
    /**
     * Gets a list of all the reservations made to the system.
     *
     * return:
     *  !null
     */
    public Collection getAllReservations() throws SQLException {
	
	ArrayList reservations = new ArrayList();
	
	ResultSet rs = findAllStmt.executeQuery();
	while (rs.next()) {
	    Reservation r = new Reservation(rs.getLong(1), rs.getString(2),
					    rs.getString(3), rs.getString(4),
					    rs.getString(5), rs.getString(6),
					    rs.getDate(7), rs.getString(8),
					    rs.getDate(9),rs.getDouble(10));
	    reservations.add(r);
	}
	
	Collections.sort(reservations);
	rs.close();
	return reservations;
    }
   

   /**
   * Get the sum of all the rental equipment.
   *
   * return:
   * >= 0.0
   */
   public double getSum() throws SQLException{

	double num = 0.0;

   	ResultSet rs = sumStmt.executeQuery();
	
	if (rs.next()){ 
		//System.out.println(rs.getString(1));
		num = rs.getDouble(1);
	}
	rs.close();

	return num;
   }

   /**
    * Get the average of all the rental equipment.
    *
    * return:
    * >= 0.00
    */
   public double getAverage() throws SQLException{


   	double num = 0.0;

	ResultSet rs = averageStmt.executeQuery();

	if (rs.next())
		num = rs.getDouble(1);

	rs.close();

	return num;

    }

    // PRIVATE methods
    
    private void initConnection () throws SQLException{

	try {
	    Class.forName("oracle.jdbc.driver.OracleDriver");
	} catch (ClassNotFoundException class_e) {
	    class_e.printStackTrace();
	}
	con = DriverManager.getConnection (dbName);
	insertCustomerStmt = con.prepareStatement(insertCustomer);
	updateCustomerStmt = con.prepareStatement(updateCustomer);
	findCustomerByEmailStmt = con.prepareStatement(selectByEmail);
	findRentalByID = con.prepareStatement(selectByID);
	deleteRentalByID = con.prepareStatement(deleteByID);
	insertRentalStmt = con.prepareStatement(insertRental);
	findConfirmationIDByRest = con.prepareStatement(selectByRest);
	insertLocationStmt = con.prepareStatement(insertLocation);
	deleteLocationStmt = con.prepareStatement(deleteLocation);
	findAllStmt = con.prepareStatement(findAll);
	maxStmt = con.prepareStatement(maxString);
	sumStmt = con.prepareStatement(sumString);
	averageStmt = con.prepareStatement(averageString);
    }
    
    /**
     * Insert a reservation into the system.
     */
    private ReservationStatus insertReservation(Customer customer, Fleet fleet) throws SQLException{
	
	ReservationStatus status;
	
	insertRentalStmt.setDouble(1, calculatePrice(customer, fleet));
	
	insertRentalStmt.setDate(2, customer.getDateFrom());
	insertRentalStmt.setString(3, customer.getHourFrom());
	insertRentalStmt.setString(4, customer.getCountryFrom());
	
	insertRentalStmt.setDate(5, customer.getDateTo());
	insertRentalStmt.setString(6, customer.getHourTo());
	insertRentalStmt.setString(7, customer.getCountryTo());
	
	insertRentalStmt.setString(8, customer.getEmail());
	insertRentalStmt.setString(9, customer.getEquipmentId());
	
	try {
	    insertRentalStmt.executeUpdate();
	    con.commit();
	} catch (SQLException sql) {
	    status = new ReservationStatus(false,000, sql.getSQLState());
	}
	
	status = new ReservationStatus(true,this.reservationExist(customer),"");
	
	return status;
    }

    /**
     * Derive the price that the customer must pay for the rental of 
     * the equipment.
     */
    
    private double calculatePrice(Customer customer, Fleet fleet) {
	
	double price = 0.00;
        EquipmentDetail eq = fleet.getEquipmentDetail(customer.getEquipmentId());
	
	price = customer.getWeeks() * eq.getPerWeek() + customer.getDays() * eq.getPerDay();
	
	return price;
	
    }
    
    /**
     * Update the customers records.
     */
    private void customerUpdate(Customer customer) throws SQLException {
	
	updateCustomerStmt.setString(1, customer.getFirstName());
	updateCustomerStmt.setString(2, customer.getLastName());
	updateCustomerStmt.setString(3, customer.getTelephone());
	updateCustomerStmt.setString(4, customer.getEmail());
	updateCustomerStmt.executeUpdate();
	
    }
    /**
     * Insert a new customer into the DB
     */
    private void customerInsert(Customer customer) throws SQLException {
	
	insertCustomerStmt.setString(1, customer.getEmail());
	insertCustomerStmt.setString(2, customer.getFirstName());
	insertCustomerStmt.setString(3, customer.getLastName());
	insertCustomerStmt.setString(4, customer.getTelephone());
	insertCustomerStmt.executeUpdate();
    }
    /**
     * Checks the database to see if this customer exist.
     *
     */
    private  boolean customerExist(String email) throws SQLException{
	
	ResultSet rs = null;		
	
	findCustomerByEmailStmt.setString(1, email);
	rs = findCustomerByEmailStmt.executeQuery();
	if (rs.next()) {
	    rs.close();
	    return true;
	}
	rs.close();
	return false;
    }
    
    /**
     * Finds out if the reservation exist for the specific customer for
     * the specific date and armories.
     *
     * return:
     * -1 if couldn't find one.
     */
    private long reservationExist(Customer customer) throws SQLException{
	
	long answer = -1;
	
	con.commit();
	findConfirmationIDByRest.setString(1, customer.getEmail());
	findConfirmationIDByRest.setString(2, customer.getEquipmentId());
	findConfirmationIDByRest.setDate(3, customer.getDateTo());
	findConfirmationIDByRest.setDate(4, customer.getDateFrom());
	
	ResultSet rs = findConfirmationIDByRest.executeQuery();
	
	if (rs.next()) {
	    answer = rs.getLong(1);
	}
	System.out.println("Customer "+customer.getEmail() +" rsv is "+answer+" ("+customer.getEquipmentId()+")");
	rs.close();
	return answer;	
    }

}
